#include <string.h>
#include <locale.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <gconf/gconf-client.h>

#include <config.h>

#ifdef GNOKII_GCONF
#include "gnokii-gconf.h"
#endif

#include "contactinfos.h"
#include "client-contactinfos-dbus.h"
#include "marshall.h"

#if HILDON == 1
#include <hildon/hildon.h>
#else
#include <hildon-widgets/hildon-program.h>
#include <hildon-widgets/hildon-banner.h>
#endif

#include <libosso.h>
#include <dbus/dbus-glib.h>
#include <bt-dbus.h>

#include "gnokii.h"

#define _(String) String

struct UICStruct {
	GtkListStore *storeNumber;
	GtkWidget *searchEntry;
	GtkWidget *treeNumber;
	GtkListStore *storeSearch;
	GtkWidget *treeSearch;
	GtkWidget *contactInfosCheckBox;
	GtkWidget *sendSMSButton;
	GtkWidget *smsLabel;
	gchar *smsText;
	osso_context_t *osso_context;
	gboolean contactInfosAvailable;
	gboolean dial;
	gboolean gsm;
	gboolean transformPlus;
	gulong sigId;
};
typedef struct UICStruct UIComponents;

static struct gn_statemachine * state;

/* Description des colonnes*/
enum
{
   NAME_COLUMN,
   NUMBER_COLUMN,
   N_COLUMNS
};

static void dial (UIComponents *uic);
static void sendSMSToTree (UIComponents *uic);
void dialNumberFromSearch(UIComponents * uic, gboolean gsm);
void dialNumber(UIComponents * uic, gchar *number);
static void sendSMS (gchar *number, gchar *text);

/* D-BUS service definitions */
#define APPLICATION_DBUS_SERVICE "phonelink"

#define GCONF_TRANSFORM_PLUS_KEY "/apps/phonelink/TransformPlusIntoDoubleZero"

static void  quit_event( UIComponents *uic, guint callback_action, GtkWidget *menu_item	) {
	GConfClient *gconf_client = NULL;

	gconf_client = gconf_client_get_default ();
	gconf_client_set_bool(gconf_client, GCONF_TRANSFORM_PLUS_KEY, uic->transformPlus, NULL);
	gconf_client_suggest_sync (gconf_client, NULL);
	g_object_unref (gconf_client);

        gn_lib_phoneprofile_free(&state);
        gn_lib_library_free();
			
	gtk_main_quit ();
	osso_deinitialize(uic->osso_context);
	g_free(uic);
					
}

/* callback to check availability of contactinfos service */
gboolean checkAvailability_cb(UIComponents *uic) {
	
	DBusGConnection *connection;
	GError *error = NULL;
	DBusGProxy *proxy;
	gboolean availability = FALSE;
	gboolean ret;

	error = NULL;
	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
	if (connection == NULL) 	{
		g_printerr ("Failed to open connection to bus: %s\n", error->message);
		g_error_free (error);
		return FALSE;
	}

	/* Create a proxy object for the "bus driver" (name "org.freedesktop.DBus") */

	proxy = dbus_g_proxy_new_for_name (connection,
				     CONTACTINFOS_DBUS_NAME,
				     CONTACTINFOS_DBUS_PATH,
				     CONTACTINFOS_DBUS_INTERFACE);


	ret = com_nokia_contactinfos_is_available (proxy, &availability, &error);

	if (ret) {
		gtk_widget_set_sensitive(uic->contactInfosCheckBox, availability);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(uic->contactInfosCheckBox), availability);
	}

	g_object_unref (proxy);
	
	return FALSE;

}

void oneResultSignal_cb(
				DBusGProxy *proxy, 
				gchar *name, 
				gchar *company, 
				guint typeInfo, 
				gchar *type, 
				gchar *value, 
				guint elementId, 
				guint valueId, 
				gint retCode, 
				UIComponents *uic) {
	GtkTreeIter iter;
	gchar *text;
					
	if (retCode == CI_NOT_FOUND) {
		text = g_strdup_printf(_("ContactInfos found nothing for %s"), (gchar *) gtk_entry_get_text(GTK_ENTRY(uic->searchEntry)));
		hildon_banner_show_information(GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), GTK_STOCK_DIALOG_INFO, text);
		g_free(text);
	}
					
	if (retCode >= CI_NO_ERROR) {
		gtk_list_store_append (uic->storeNumber, &iter);
		gtk_list_store_set (uic->storeNumber, &iter, 
					NAME_COLUMN, name, 
					NUMBER_COLUMN, value, 
					-1);
	}
	
	if (retCode < CI_NO_ERROR || (elementId == CI_LAST_ELEMENT_ID && valueId==CI_LAST_VALUE_ID)) {
		dbus_g_proxy_disconnect_signal(proxy, CONTACTINFOS_DBUS_RESULTS_SIGNAL,
			G_CALLBACK(oneResultSignal_cb), uic);

		g_object_unref (proxy);
	}
	
	if (elementId == CI_LAST_ELEMENT_ID && valueId==CI_LAST_VALUE_ID) {
		if (uic->dial) {
			dial(uic);
		} else {
			sendSMSToTree(uic);
		}
	}

}

void on_menu_quit(GtkMenuItem * menuitem, UIComponents *uic) {
	quit_event( uic, 0, NULL);
}

void on_menu_about(GtkMenuItem * menuitem, UIComponents *uic) {
	gtk_show_about_dialog(GTK_WIDGET(gtk_widget_get_toplevel(menuitem)),
		"version", VERSION,
		"website", "https://garage.maemo.org/projects/phonelink/",
		"website-label", _("Garage Page @ Maemo"),
		"authors", g_strsplit("Fred Lef\xc3\xa9v\xc3\xa8re-Laoide\n", "\n", -1),
		"logo", gdk_pixbuf_new_from_file(ICONFILE, NULL),
		NULL);
}

void on_menu_transform(GtkCheckMenuItem *checkMenuitem, UIComponents *uic) {
	uic->transformPlus = !(uic->transformPlus);
}

GtkMenu *create_menus(UIComponents *uic) {
	GtkWidget *menuBar;
	GtkWidget *menuGroup;
	GtkWidget *menuItem;
	GtkWidget *image;
	
	menuBar = gtk_menu_new ();
	
	menuGroup = gtk_menu_new();
	menuItem = gtk_menu_item_new_with_label (_("File"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItem), menuGroup);
	gtk_menu_append(menuBar, menuItem);
	
	menuItem = gtk_image_menu_item_new_with_label (_("Quit"));
	image = gtk_image_new_from_stock (GTK_STOCK_QUIT, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuItem), image);
	g_signal_connect (G_OBJECT (menuItem), "activate", G_CALLBACK (on_menu_quit), (gpointer) uic);
	gtk_menu_append(menuGroup, menuItem);
	
	menuGroup = gtk_menu_new();
	menuItem = gtk_menu_item_new_with_label (_("Edit"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItem), menuGroup);
	gtk_menu_append(menuBar, menuItem);
	
	menuItem = gtk_check_menu_item_new_with_label (_("Transform + into 00"));
	g_signal_connect (G_OBJECT (menuItem), "toggled", G_CALLBACK (on_menu_transform), (gpointer) uic);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menuItem), uic->transformPlus);
	gtk_menu_append(menuGroup, menuItem);
	
	menuGroup = gtk_menu_new();
	menuItem = gtk_menu_item_new_with_label (_("Help"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItem), menuGroup);
	gtk_menu_append(menuBar, menuItem);
	
	menuItem = gtk_image_menu_item_new_with_label (_("About ..."));
	image = gtk_image_new_from_stock (GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuItem), image);
	g_signal_connect (G_OBJECT (menuItem), "activate", G_CALLBACK (on_menu_about), (gpointer) uic);
	gtk_menu_append(menuGroup, menuItem);
	
	return GTK_MENU(menuBar);
}



void getDBusNumbers(UIComponents *uic, gboolean dial, gboolean gsm) {
	DBusGConnection *connection;
	GError *error = NULL;
	DBusGProxy *proxy;
	gboolean availability;
	gboolean multipleSel=FALSE;
	guint typeInfo=CI_ALL_PHONE_NUMBER;;
	
	uic->dial = dial;
	uic->gsm = gsm;
	
	gtk_list_store_clear(uic->storeNumber);

	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
	if (connection == NULL) {
		g_printerr ("Failed to open connection to bus: %s\n", error->message);
		g_error_free (error);
		return;
	}

	/* Create a proxy object for the "bus driver" (name "org.freedesktop.DBus") */
	proxy = dbus_g_proxy_new_for_name (connection,
				     CONTACTINFOS_DBUS_NAME,
				     CONTACTINFOS_DBUS_PATH,
				     CONTACTINFOS_DBUS_INTERFACE);


	dbus_g_object_register_marshaller(
		dbus_glib_marshal_contactinfos_VOID__STRING_STRING_UINT_STRING_STRING_UINT_UINT_INT,
		G_TYPE_NONE,
		G_TYPE_STRING,
		G_TYPE_STRING,
		G_TYPE_UINT,
		G_TYPE_STRING,
		G_TYPE_STRING,
		G_TYPE_UINT,
		G_TYPE_UINT,
		G_TYPE_INT,
		G_TYPE_INVALID);

	dbus_g_proxy_add_signal(proxy,
		CONTACTINFOS_DBUS_RESULTS_SIGNAL,
		G_TYPE_STRING,
		G_TYPE_STRING,
		G_TYPE_UINT,
		G_TYPE_STRING,
		G_TYPE_STRING,
		G_TYPE_UINT,
		G_TYPE_UINT,
		G_TYPE_INT,
		G_TYPE_INVALID);

	dbus_g_proxy_connect_signal(proxy, CONTACTINFOS_DBUS_RESULTS_SIGNAL,
		G_CALLBACK(oneResultSignal_cb), uic, NULL);
	
	multipleSel = !dial;

	/* Start DBus Call */
	com_nokia_contactinfos_get_contact_infos (
								proxy, 
								typeInfo, 
								gtk_entry_get_text(GTK_ENTRY(uic->searchEntry)), 
								TRUE, 
								TRUE, 
								multipleSel, 
								0, 
								&availability, 
								&error);
/*
	if (error) {
		res  = g_strdup_printf("Error\n  Code = %d\n  Message = %s\n", error->code, error->message);
		gtk_text_buffer_insert_at_cursor (appData->buffer, res, -1);
		g_free(res);
	} else {
		if (availability) {
			res  = g_strdup_printf("callDbusSignal_cb No Error\nWaiting for an answer ...\n");
			gtk_text_buffer_insert_at_cursor (appData->buffer, res, -1);
			g_free(res);
		} else {
			res  = g_strdup_printf("callDbusSignal_cb No Error\nForget it ...\n");
			gtk_text_buffer_insert_at_cursor (appData->buffer, res, -1);
			g_free(res);
		}
	}
*/
}

void dialDBus_cb(GtkButton *widget, UIComponents *uic) {
	gboolean availability;
	
	availability = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(uic->contactInfosCheckBox));
	
	if (availability) {
		getDBusNumbers(uic, TRUE, TRUE);
	} else {
		/* dial the number from the searchEntry */
		dialNumberFromSearch(uic, TRUE);
	}
	
}

/* another callback */
static gboolean delete_event( GtkWidget *widget, GdkEvent *event, gpointer data ) {
	quit_event(data, 0, NULL);
	return TRUE;
}

static gchar * normalizeNumber( gchar *number, gboolean transformPlus) {

	if (!number) {
		return number;
	}

	gchar * normalizedNumber = (gchar *) g_malloc(strlen(number)+1);
	guint i = 0;
	guint j = 0;
	g_strstrip(number);
	if (transformPlus && number[0] == '+') {
		normalizedNumber[0] = '0';
		normalizedNumber[1] = '0';
		j = 2;
		i = 1;
	}

	for (; number[i]; i++) {
		if (g_ascii_isdigit(number[i]) || number[i] == '+') {
			normalizedNumber[j++] = number[i];
		}
	}
	normalizedNumber[j] = '\0';
	return normalizedNumber;
}


static gchar * normalizeNumber4Sip( gchar *number, gboolean transformPlus ) {

	gchar * num = normalizeNumber(number, transformPlus);
	gchar * res = g_strconcat("sip:", num, NULL); /*, "@freephonie.net" */
	g_free(num);
	return res;
}
	
static void dialSip_cb (GtkButton *widget, UIComponents *uic) {
	gboolean availability;
	
	availability = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(uic->contactInfosCheckBox));
	
	if (availability) {
		getDBusNumbers(uic, TRUE, FALSE);
	} else {
		/* dial the number from the searchEntry */
		dialNumberFromSearch(uic, FALSE);
	}
	
}

static guint dialSip(gchar * number, gchar ** message, UIComponents *uic) {
	
/* initialise the connection */
	osso_rpc_t retval;
	osso_return_t ret;
	ret = osso_rpc_run(uic->osso_context,
			"com.nokia.gizmo",
			"/com/nokia/gizmo",
			"com.nokia.gizmo",
			"callto", 
			&retval,
			DBUS_TYPE_STRING, number,
			DBUS_TYPE_INVALID);
	osso_rpc_free_val(&retval);
	if (ret !=  OSSO_OK) {
		*message=g_strdup_printf("Can't dial Sip phone through DBus : %d", ret);
		return 1;
	}
	return 0;
}

static guint dialGsm(gchar *number, gchar **message) {

	guint ret = 0;

	gn_data gdata;
        gn_error error;
        gn_call_info call_info;
        int call_id;
				
	/* initialise the connection */
	if ((error = gn_lib_phone_open(state)) != GN_ERR_NONE) {
		g_print("Telephone interface init failed: %s Quitting.\n", gn_error_print(error));
		return 1;
	}

	memset(&call_info, 0, sizeof(call_info));
	snprintf(call_info.number, sizeof(call_info.number), "%s", number);
	call_info.type = GN_CALL_Voice;
	call_info.send_number = GN_CALL_Default;
	gn_data_clear(&gdata);
	gdata.call_info = &call_info;
        if ((error = gn_call_dial(&call_id, &gdata, state)) != GN_ERR_NONE) {
		*message=g_strdup_printf("Can't dial the phone : %s", gn_error_print(error));
		ret = 2;
	} else {
		g_print("Dialled call, id: %d (lowlevel id: %d)\n", call_id, call_info.call_id);
	}
	 /* tear down the connection */
	gn_lib_phone_close(state);
	return ret;
																			 
}

void dialNumberFromSearch(UIComponents * uic, gboolean gsm) {
	gchar *number;
	
	uic->gsm = gsm;
	number = (gchar *) gtk_entry_get_text(GTK_ENTRY(uic->searchEntry));
	dialNumber(uic, number);
}

void dialNumber(UIComponents * uic, gchar *number) {
	gchar *normalizedNumber;
	gchar *sipNormalizedNumber;
	gchar *text;
	
	if (number && number[0] != '\0') {
		normalizedNumber = normalizeNumber(number, uic->transformPlus);
		if (normalizedNumber == NULL || normalizedNumber[0] == '\0') {
			text = g_strdup_printf("Number %s is not Valid !\n", number);
			hildon_banner_show_information(GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), GTK_STOCK_DIALOG_ERROR, text);
			g_free(text);
			g_free(normalizedNumber);
			return;
		}
		text = g_strdup_printf("Dialing %s ...", normalizedNumber);
		hildon_banner_show_information(GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), GTK_STOCK_CONNECT, text);
		g_free(text);
		text=NULL;
		guint ret = 0;
		if (uic->gsm == TRUE) {
			ret = dialGsm(normalizedNumber, &text);
		} else {
			sipNormalizedNumber = normalizeNumber4Sip(normalizedNumber, uic->transformPlus);
			ret = dialSip(sipNormalizedNumber, &text, uic);
			g_free(sipNormalizedNumber);
		}
		if (ret != 0) {
			hildon_banner_show_information(GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), GTK_STOCK_DIALOG_ERROR, text);
		}
		g_free(text);
		g_free(normalizedNumber);
	}
}

static void dial (UIComponents * uic) {
	
	gchar *number;
	GtkTreeIter iter;
	GtkTreeModel *model;

	/* get the data */
	model = GTK_TREE_MODEL(uic->storeNumber);
	if (gtk_tree_model_get_iter_first(model, &iter)) {
		gtk_tree_model_get(model, &iter, NUMBER_COLUMN, &number, -1);
		dialNumber(uic, number);
		g_free(number);
	}
}

static void printData(gpointer data, gpointer data2) {
	g_print("%s; ", (gchar *) data);
}

static gchar * getSMSText(UIComponents * uic) {
	
	if (!uic->smsText || strlen(uic->smsText) == 0) {
		return NULL;
	}
	return uic->smsText;
}

void smsSetCount(UIComponents * uic, gint count) {
	gchar *lib;
	lib = g_strdup_printf("SMS : %d car.", count);
	gtk_label_set_text(GTK_LABEL(uic->smsLabel), lib);
	g_free(lib);
}

/* Called when something is typed into the sms zone */
void smsBufferChanged_cb(GtkTextBuffer *textBuffer, UIComponents * uic) {
	/* Enable the SMS Button */
	GtkTextIter start, end;
	int count = 0;
	
	gtk_text_buffer_set_modified (textBuffer, FALSE);
	gtk_text_buffer_get_bounds(textBuffer, &start, &end);
	uic->smsText = gtk_text_buffer_get_text(textBuffer, &start, &end, TRUE);

	if (!uic->smsText || (count = strlen(uic->smsText)) == 0) {
		gtk_widget_set_sensitive(uic->sendSMSButton, FALSE);
	} else {
		gtk_widget_set_sensitive(uic->sendSMSButton, TRUE);
	}
	smsSetCount(uic, count);
	
}

/*
static gchar * getPhoneInfos(GtkWidget *window) {
	gchar *macAddr;
	gchar *model;
	gchar *text;
	GtkWidget *dialog;
	gint result;
	GtkWidget *hbox = NULL;
	GtkWidget *macEntry;
	GtkWidget *modelComboEntry;
	GtkWidget *btn_scan = NULL;
	ScanInfo scan_info = {0};
	
	dialog = gtk_dialog_new_with_buttons ("Gnokii Config",
			                       GTK_WINDOW(window),
						GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
						GTK_STOCK_OK,
						GTK_RESPONSE_ACCEPT,
						GTK_STOCK_CANCEL,
						GTK_RESPONSE_REJECT,
						NULL);
	hbox = gtk_hbox_new(FALSE, 4);
	gtk_box_pack_start(GTK_BOX(hbox),
                macEntry = gtk_entry_new(),
                TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox),
                btn_scan = gtk_button_new_with_label("Scan..."),
                FALSE, FALSE, 0);
	gtk_container_add (GTK_CONTAINER(((GtkDialog *)dialog)->vbox), hbox);
	modelComboEntry = gtk_combo_box_entry_new_text();
	gtk_combo_box_append_text(GTK_COMBO_BOX(modelComboEntry),"AT");
	gtk_combo_box_append_text(GTK_COMBO_BOX(modelComboEntry),"6230i");
	gtk_combo_box_append_text(GTK_COMBO_BOX(modelComboEntry),"symbian");
	gtk_combo_box_append_text(GTK_COMBO_BOX(modelComboEntry),"...");
	gtk_container_add (GTK_CONTAINER(((GtkDialog *)dialog)->vbox), modelComboEntry);
	gtk_window_set_default_size(GTK_WINDOW(dialog), 600, 300);

	/* Connect signals. */
  /*
	scan_info.settings_dialog = dialog;
        scan_info.txt_rcvr_mac = macEntry;
        g_signal_connect(G_OBJECT(btn_scan), "clicked",
                         G_CALLBACK(scan_bluetooth), &scan_info);

	
	gtk_widget_show_all (GTK_WIDGET(dialog));
	
	result = gtk_dialog_run (GTK_DIALOG (dialog));
	switch (result) {
		case GTK_RESPONSE_ACCEPT:
			macAddr = gtk_entry_get_text(GTK_ENTRY(macEntry));
			model = gtk_combo_box_get_active_text(GTK_COMBO_BOX(modelComboEntry));
			text = g_strjoin("|", macAddr, model, NULL);
			break;
		default:
			text = NULL;
			break;
	}
	gtk_widget_destroy (dialog);
	
	return text;
}
*/

static gchar * cleanNumber(gchar * number) {
	guint i, j;

	if (number) {
		for (i = j = 0; number[i]; i++) {
			if (number[i] != ' ') {
				number[j++] = number[i];
			}
		}
		number[j] = '\0';
	}
	return number;
}

/*
static void getPhoneInfos_cb(GtkButton *widget, gpointer data) {
	gchar *text;
	text = getPhoneInfos(gtk_widget_get_toplevel(GTK_WIDGET(widget)));
	if (text) {
		g_print("%s\n", text);
		g_free(text);
	}
}
*/

static void showMessageSendSMS(GtkWidget *widget, gchar *number) {
	gchar *text;
	text = g_strdup_printf("Sending SMS to %s ...", number);
	hildon_banner_show_information(GTK_WIDGET(gtk_widget_get_toplevel(widget)), GTK_STOCK_DND, text);
	g_free(text);
}

static void sendSMS_cb (GtkButton *widget, UIComponents *uic) {

	gchar *normalizedNumber;
	gchar *text;
	gn_error error;
	gboolean availability;
	
	availability = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(uic->contactInfosCheckBox));
	
	if (availability) {
		getDBusNumbers(uic, FALSE, TRUE);
	} else {
		/* dial the number from the searchEntry */
		normalizedNumber=normalizeNumber((gchar *) gtk_entry_get_text(GTK_ENTRY(uic->searchEntry)), uic->transformPlus);
		text = getSMSText(uic);
		if (text) {
			showMessageSendSMS(GTK_WIDGET(widget), normalizedNumber);
			/* initialise the connection */
			if ((error = gn_lib_phone_open(state)) != GN_ERR_NONE) {
				g_print("Telephone interface init failed: %s Quitting.\n", gn_error_print(error));
				return;
			}
			
			sendSMS(normalizedNumber, text);
			g_free(text);

			/* tear down the connection */
			gn_lib_phone_close(state);
		}
		g_free(normalizedNumber);
	}

}

static void sendSMS (gchar *number, gchar *text) {
	gn_sms sms;
	gn_error error;
	gn_data data;

	gn_data_clear(&data);
	
	/* The maximum length of an uncompressed concatenated short message is
	 * 	   255 * 153 = 39015 default alphabet characters */
	int input_len, i, curpos = 0;

	g_printf("sendSMS(number = %s, text = %s)\n", number, text);
	
	input_len = GN_SMS_MAX_LENGTH;

	/* The memory is zeroed here */
	gn_sms_default_submit(&sms);

	memset(&sms.remote.number, 0, sizeof(sms.remote.number));
	strncpy(sms.remote.number, number, sizeof(sms.remote.number) - 1);
	if (sms.remote.number[0] == '+') {
		sms.remote.type = GN_GSM_NUMBER_International;
	} else {
		sms.remote.type = GN_GSM_NUMBER_Unknown;
	}
	
	data.message_center = calloc(1, sizeof(gn_sms_message_center));
	data.message_center->id = 1;
	if (gn_sm_functions(GN_OP_GetSMSCenter, &data, state) == GN_ERR_NONE) {
		strcpy(sms.smsc.number, data.message_center->smsc.number);
		sms.smsc.type = data.message_center->smsc.type;
	} else {
		g_print("Cannot read the SMSC number from your phone. If the sms send will fail, please use --smsc option explicitely giving the number.\n");
	}
	free(data.message_center);

	if (!sms.smsc.type) sms.smsc.type = GN_GSM_NUMBER_Unknown;

	i = strlen(text);
	strncpy(sms.user_data[curpos].u.text, text, input_len);
	sms.user_data[curpos].u.text[i] = '\0';
	sms.user_data[curpos].length = i;
	if (sms.user_data[curpos].length < 1) {
		g_print("Empty message. Quitting.\n");
		g_free(number);
		return ;
	}
	sms.user_data[curpos].type = GN_SMS_DATA_Text;
	if (!gn_char_def_alphabet(sms.user_data[curpos].u.text)) {
		sms.dcs.u.general.alphabet = GN_SMS_DCS_UCS2;
	}
	sms.user_data[++curpos].type = GN_SMS_DATA_None;
	data.sms = &sms;

	/* Send the message. */
	error = gn_sms_send(&data, state);

	if (error == GN_ERR_NONE) {
		g_print("Send succeeded!\n");
	} else {
		g_print("SMS Send failed (%s)\n", gn_error_print(error));
	}

}

gboolean sendSMSToIter (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, UIComponents *uic) {
	gchar *text;
	gchar *number;
	gchar *normalizedNumber;
	
	text=getSMSText(uic);
			
	gtk_tree_model_get(model, iter, NUMBER_COLUMN, &number, -1);
	normalizedNumber = normalizeNumber(number, uic->transformPlus);
	showMessageSendSMS(GTK_WIDGET(uic->searchEntry), normalizedNumber);
	sendSMS (normalizedNumber, text);
	g_free(normalizedNumber);
	g_free(text);
	return FALSE;
}

static void sendSMSToTree (UIComponents *uic) {
	gn_error error;
	
	/* initialise the connection */
	if ((error = gn_lib_phone_open(state)) != GN_ERR_NONE) {
		g_print("Telephone interface init failed: %s Quitting.\n", gn_error_print(error));
		return;
	}
	
	gtk_tree_model_foreach (GTK_TREE_MODEL(uic->storeNumber), (GtkTreeModelForeachFunc)sendSMSToIter, (gpointer)uic);

	/* tear down the connection */
	gn_lib_phone_close(state);
}

gboolean clearSMSTextView_cb(GtkTextView *smsTextView, GdkEventFocus *event, UIComponents *uic) {
	GtkTextBuffer *buffer;
	buffer = gtk_text_view_get_buffer(smsTextView);
	gtk_text_buffer_set_text(buffer, "", -1);
	gtk_text_buffer_set_modified (buffer, FALSE);
	g_signal_handler_disconnect(G_OBJECT(smsTextView), uic->sigId);
	return FALSE;
}


/**
 * Create the menu items needed for the drop down menu.
 */
/*
void menu_init(HildonWindow *window) {
    /* Create needed handles. */
/*
    GtkMenu *menu;
    GtkWidget *submenu;
    GtkWidget *menu_item;
    printf("%s()\n", __PRETTY_FUNCTION__);

    /* Get the menu of our view. */
/*
    menu = GTK_MENU(gtk_menu_new());

    /* Create the menu items. */

    /* The "Config" submenu. */
/*
    gtk_menu_append(menu, menu_item
            = gtk_menu_item_new_with_label("configuration"));


    /* We need to show menu items. */
/*
    gtk_widget_show_all(GTK_WIDGET(menu));

    hildon_window_set_menu(HILDON_WINDOW(window), menu);

    /* Connect the "Route" signals. */
/*
    g_signal_connect(G_OBJECT(menu_item), "activate",
                      G_CALLBACK(getPhoneInfos_cb), NULL);

}
*/

int main( int argc, char *argv[] ) {
	    /* GtkWidget is the storage type for widgets */
	GtkWidget *button;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *table;
	GtkWidget *scrolled_window;
	GtkWidget *smsTextView;
	GtkTextBuffer *buffer;
	GConfClient *gconf_client = NULL;
	gboolean gnokiiFile = FALSE;
	
#ifdef GNOKII_GCONF
	gchar ** gnokiiCfg;
#endif	
	
	HildonProgram *app;
	HildonWindow *appview;
	
	UIComponents *uic = g_new0(UIComponents, 1);

	/* This is called in all GTK applications. Arguments are parsed
	 * from the command line and are returned to the application. */
	gtk_init (&argc, &argv);

	/* Create a new window */
        app = HILDON_PROGRAM( hildon_program_get_instance() );
        g_set_application_name(_("PhoneLink" ));
	
        /* Create HildonAppView and set it to HildonApp */
        appview = HILDON_WINDOW( hildon_window_new());
        hildon_program_add_window( app, appview );

	g_type_init ();

	/* Read GConf Info */
	gconf_client = gconf_client_get_default ();
	uic->transformPlus = gconf_client_get_bool(gconf_client, GCONF_TRANSFORM_PLUS_KEY, NULL);

	hildon_window_set_menu (appview, create_menus (uic));

/* Gnokii : read the config */
	gn_error err;
#ifdef GNOKII_GCONF
	if (!isGnokiiRcFileUsed(gconf_client, NULL)) {
		gnokiiCfg = getGnokiiCfg(gconf_client, NULL);
		gnokiiFile = TRUE;
		state = g_new0(struct gn_statemachine,1);
		if ((err = gn_cfg_memory_read ((char **)gnokiiCfg)) != GN_ERR_NONE) {
			hildon_banner_show_informationf(GTK_WIDGET(appview), GTK_STOCK_DIALOG_ERROR, _("Error reading gnokii configuration %s\n%s\n%s\n%s\n%s"), gn_error_print(err), gnokiiCfg[0], gnokiiCfg[1], gnokiiCfg[2], gnokiiCfg[3]);
			gnokiiFile = FALSE;
		} else if ((err = gn_cfg_phone_load(NULL, state)) != GN_ERR_NONE) {
			fprintf(stderr, "%s\n", gn_error_print(err));
			hildon_banner_show_informationf(GTK_WIDGET(appview), GTK_STOCK_DIALOG_ERROR, _("gn_cfg_phone_load %s"), gn_error_print(err));
			gnokiiFile = FALSE;
		}
	}
#endif
	if (!gnokiiFile) {
		if ((err = gn_lib_phoneprofile_load(NULL, &state)) != GN_ERR_NONE) {
			fprintf(stderr, "%s\n", gn_error_print(err));
			hildon_banner_show_information(GTK_WIDGET(appview), GTK_STOCK_DIALOG_ERROR,  gn_error_print(err));
		}
	}

	g_object_unref (gconf_client);
	
	g_print("avant osso init\n");

	/* Initialize maemo application */
	uic->osso_context = osso_initialize(APPLICATION_DBUS_SERVICE, VERSION, FALSE, NULL);

	g_print("apres osso_init\n");
	            
	/* Check that initialization was ok */
	if (uic->osso_context == NULL) {
		 hildon_banner_show_information(GTK_WIDGET(appview), GTK_STOCK_DIALOG_ERROR, "osso_initialize error\n");
		return OSSO_ERROR;
	}
	
	
	/* Here we just set a handler for delete_event that immediately
	*      * exits GTK. */
	g_signal_connect (G_OBJECT (appview), "delete_event",
		      G_CALLBACK (delete_event), (gpointer) uic);

	// la boite gnrale
	table = gtk_table_new(9, 5, TRUE);
	gtk_container_add (GTK_CONTAINER (appview), table);
	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
	
	/* Add a field & 2 buttons */
	uic->searchEntry = gtk_entry_new();
	gtk_table_attach_defaults( GTK_TABLE(table), uic->searchEntry, 0, 3, 0, 1);

	/* Buttons */
	button = gtk_button_new_with_label ("Dial");
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	g_signal_connect (G_OBJECT (button), "clicked",
	                  G_CALLBACK (dialDBus_cb), (gpointer) uic);
	gtk_table_attach_defaults(GTK_TABLE(table), button, 3, 5, 0, 1);
	gtk_widget_grab_default(button);
	
	uic->contactInfosCheckBox = gtk_check_button_new_with_label ("Use ContactInfos");
	gtk_widget_set_sensitive(uic->contactInfosCheckBox, FALSE);
	gtk_table_attach_defaults( GTK_TABLE(table), uic->contactInfosCheckBox, 3, 5, 1, 2);

	uic->sendSMSButton = gtk_button_new_with_label ("Send Sms");
	g_signal_connect (G_OBJECT (uic->sendSMSButton), "clicked",
			G_CALLBACK (sendSMS_cb), (gpointer) uic);
	gtk_widget_set_sensitive(uic->sendSMSButton, FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table), uic->sendSMSButton, 3, 4, 2, 3);

	button = gtk_button_new_with_label ("Dial Sip");
	g_signal_connect (G_OBJECT (button), "clicked",
	                  G_CALLBACK (dialSip_cb), (gpointer) uic);
	gtk_table_attach_defaults(GTK_TABLE(table), button, 4, 5, 2, 3);
			
	// la liste des numros choisis
	uic->storeNumber = gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);
	
	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		
	uic->treeNumber = gtk_tree_view_new_with_model (GTK_TREE_MODEL (uic->storeNumber));
	
	GTK_WIDGET_UNSET_FLAGS(uic->treeNumber, GTK_CAN_FOCUS);
	gtk_container_add(GTK_CONTAINER(scrolled_window), uic->treeNumber);
	gtk_table_attach_defaults( GTK_TABLE(table), scrolled_window, 0, 3, 1, 4);
	
        renderer = gtk_cell_renderer_text_new ();
        column = gtk_tree_view_column_new_with_attributes ("Number", renderer, "text", NUMBER_COLUMN, NULL);
        gtk_tree_view_append_column (GTK_TREE_VIEW (uic->treeNumber), column);
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Name", renderer, "text", NAME_COLUMN, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (uic->treeNumber), column);
	
	/* textEdit for SMSes */
	uic->smsLabel = gtk_label_new (NULL);
	smsSetCount(uic, 0);
	gtk_table_attach_defaults( GTK_TABLE(table), uic->smsLabel, 3, 5, 3, 4);
	
	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

	smsTextView = gtk_text_view_new();
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(smsTextView), GTK_WRAP_WORD_CHAR);
	
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(smsTextView));
	gtk_text_buffer_set_text(buffer, _("Enter SMS Text here"), -1);
	gtk_text_buffer_set_modified (buffer, FALSE);
	g_signal_connect (G_OBJECT (buffer), "modified-changed", G_CALLBACK (smsBufferChanged_cb), (gpointer) uic);

	uic->sigId = g_signal_connect (G_OBJECT (smsTextView), "focus-in-event", G_CALLBACK (clearSMSTextView_cb), (gpointer) uic);

	gtk_container_add(GTK_CONTAINER(scrolled_window), smsTextView);

	gtk_table_attach_defaults( GTK_TABLE(table), scrolled_window, 0, 5, 4, 9);
	
	gtk_widget_show_all (GTK_WIDGET(appview));
	
	/* check availability and show or hide the Buttons */
	g_idle_add ((GSourceFunc)checkAvailability_cb, uic);

	/* Rest in gtk_main and wait for the fun to begin! */
	gtk_main ();

	return 0;
}

